<?php

namespace Drupal\ts_dx\Services\Context;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\CurrentRouteMatch;

/**
 * Provides tools for context.
 *
 * @package Drupal\ts_dx\Services\Context
 */
class ContextTools
{

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'ts_dx.context_tools';

  /**
   * Current route match.
   *
   * @var CurrentRouteMatch
   */
  protected CurrentRouteMatch $currentRouteMatch;

  /**
   * Current context entity.
   *
   * @var EntityInterface|bool|null
   */
  protected $currentContextEntity = FALSE;

  /**
   * Constructor.
   *
   * @param CurrentRouteMatch $currentRouteMatch
   */
  public function __construct(CurrentRouteMatch $currentRouteMatch)
  {
    $this->currentRouteMatch = $currentRouteMatch;
  }


  /**
   * Singleton quick access.
   *
   * @return static
   *   The singleton.
   */
  public static function me()
  {
    return \Drupal::service(static::SERVICE_NAME);
  }


  /**
   * Return the entity of the current page.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The current context entity.
   */
  public function getCurrentContextEntity()
  {
    if ($this->currentContextEntity === FALSE) {
      $routeParameters = $this->currentRouteMatch->getParameters()->all();
      foreach ($routeParameters as $parameter) {
        if ($parameter instanceof EntityInterface) {
          $this->currentContextEntity = $parameter;
        }
      }
      $this->currentContextEntity = $this->currentContextEntity ?: null;
    }

    return $this->currentContextEntity;
  }

}
